# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess
import tempfile
import os
import time
import pytest

@pytest.mark.parametrize('filename', [f for f in os.listdir('examples') if f.endswith('.ipynb')])
def test_notebook(filename):
    path = os.path.join('examples', filename)
    with tempfile.NamedTemporaryFile(suffix=".ipynb") as fout:
        args = ["jupyter", "nbconvert", "--to", "notebook", "--execute",
                "--ExecutePreprocessor.timeout=1000",
                "--output", fout.name, path]
        subprocess.check_call(args)



import unittest
import os
import lale
import urllib
import pandas as pd
from sklearn.externals import joblib
from sklearn.linear_model import LogisticRegression as LR
from sklearn.tree import DecisionTreeClassifier as Tree
from sklearn.neighbors import KNeighborsClassifier as KNN


class TestAutoAIOutputConsumption(unittest.TestCase):

    pickled_model_path = "credit_risk.pickle"
    pickled_model_url = "https://github.com/pmservice/wml-sample-models/raw/master/autoai/credit-risk-prediction/model/credit_risk.pickle"
    train_csv_path = "german_credit_data_biased_training.csv"
    train_csv_url = "https://raw.githubusercontent.com/pmservice/wml-sample-models/master/autoai/credit-risk-prediction/data/german_credit_data_biased_training.csv"
    training_df = None
    model = None
    prefix_model = None
    refined_model = None
    pipeline_content = None
    pp_pipeline: lale.operators.TrainablePipeline = None

    @classmethod
    def setUp(cls) -> None:
        urllib.request.urlretrieve(cls.pickled_model_url, cls.pickled_model_path)
        urllib.request.urlretrieve(cls.train_csv_url, cls.train_csv_path)
        TestAutoAIOutputConsumption.training_df = pd.read_csv(TestAutoAIOutputConsumption.train_csv_path)

    def test_01_load_pickled_model(self):
        try:
            TestAutoAIOutputConsumption.model = joblib.load(TestAutoAIOutputConsumption.pickled_model_path)
            print(type(TestAutoAIOutputConsumption.model))
            print(TestAutoAIOutputConsumption.model)
        except Exception as e:
            assert False, f"Exception was thrown during model pickle: {e}"

    def test_02_predict_on_model(self):
        check_X = TestAutoAIOutputConsumption.training_df.drop(['Risk'], axis=1).values
        x = check_X
        try:
            pred = TestAutoAIOutputConsumption.model.predict(x)
            assert len(pred) == len(x), f"Prediction has returned unexpected number of rows {len(pred)} - expected {len(x)}"
        except Exception as e:
            assert False, f"Exception was thrown during model prediction: {e}"

    def test_03_print_pipeline(self):
        lale_pipeline = TestAutoAIOutputConsumption.model
        TestAutoAIOutputConsumption.pipeline_content = lale_pipeline.pretty_print()
        assert type(TestAutoAIOutputConsumption.pipeline_content) == str
        assert len(TestAutoAIOutputConsumption.pipeline_content) > 0
        assert "lale.wrap_imported_operators()" in TestAutoAIOutputConsumption.pipeline_content

    def test_04_execute_pipeline(self):
        try:
            with open("pp_pipeline.py", "w") as pipeline_f:
                pipeline_f.write(TestAutoAIOutputConsumption.pipeline_content)

            import importlib.util
            import lale.operators

            spec = importlib.util.spec_from_file_location("pp_pipeline", "pp_pipeline.py")
            pipeline_module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(pipeline_module)
            TestAutoAIOutputConsumption.pp_pipeline = pipeline_module.pipeline
            assert(isinstance(TestAutoAIOutputConsumption.pp_pipeline, lale.operators.TrainablePipeline))
        except Exception as e:
            assert False, f"{e}"
        finally:
            try:
                os.remove("pp_pipeline.py")
            except:
                print("Couldn't remove pp_pipeline.py file")

    def test_05_train_pretty_print_pipeline(self):
        train_X = TestAutoAIOutputConsumption.training_df.drop(['Risk'], axis=1).values
        train_y = TestAutoAIOutputConsumption.training_df.Risk.values

        TestAutoAIOutputConsumption.pp_pipeline.fit(train_X, train_y)

    def test_06_predict_on_pretty_print_pipeline(self):
        check_X = TestAutoAIOutputConsumption.training_df.drop(['Risk'], axis=1).values
        x = check_X
        try:
            pred = TestAutoAIOutputConsumption.pp_pipeline.predict(x)
            assert len(pred) == len(x), f"Prediction on pretty print model has returned unexpected number of rows {len(pred)} - expected {len(x)}"
        except Exception as e:
            assert False, f"Exception was thrown during pretty print model prediction: {e}"

    def test_07_convert_model_to_lale(self):
        import lale.operators
        try:
            lale_pipeline = TestAutoAIOutputConsumption.model
            TestAutoAIOutputConsumption.prefix_model = lale_pipeline.remove_last().freeze_trainable()
            assert(isinstance(TestAutoAIOutputConsumption.prefix_model, lale.operators.TrainablePipeline))
        except Exception as e:
            assert False, f"Exception was thrown during model prediction: {e}"

    def test_08_refine_model_with_lale(self):
        from lale.lib.lale import Hyperopt
        from lale import wrap_imported_operators

        wrap_imported_operators()
        try:
            print(f"prefix_model is of type: {type(TestAutoAIOutputConsumption.prefix_model)}")
            print(f"LR is of type {type(LR)}")
            # This is for classifiers, regressors needs to have different operators & different scoring metrics (e.g 'r2')
            new_model = TestAutoAIOutputConsumption.prefix_model >> (LR | Tree | KNN)
            train_X = TestAutoAIOutputConsumption.training_df.drop(['Risk'], axis=1).values
            train_y = TestAutoAIOutputConsumption.training_df['Risk'].values
            hyperopt = Hyperopt(estimator=new_model, cv=2, max_evals=3, scoring='roc_auc')
            hyperopt_pipelines = hyperopt.fit(train_X, train_y)
            TestAutoAIOutputConsumption.refined_model = hyperopt_pipelines.get_pipeline()
        except Exception as e:
            assert False, f"Exception was thrown during model refinery: {e}"

    def test_09_predict_refined_model(self):
        check_X = TestAutoAIOutputConsumption.training_df.drop(['Risk'], axis=1).values
        x = check_X
        try:
            pred = TestAutoAIOutputConsumption.refined_model.predict(x)
            assert len(pred) == len(
                x), f"Prediction on refined model has returned unexpected number of rows {len(pred)} - expected {len(x)}"
        except Exception as e:
            assert False, f"Exception was thrown during refined model prediction: {e}"



import unittest
from lale.grammar import Grammar
from lale.operators import make_choice, PlannedOperator, PlannedPipeline, TrainedOperator
from lale import wrap_imported_operators

from lale.lib.sklearn import LogisticRegression as LR
from lale.lib.sklearn import KNeighborsClassifier as KNN
from lale.lib.sklearn import PCA
from lale.lib.sklearn import StandardScaler as Scaler
from lale.lib.sklearn import AdaBoostClassifier as Boost
from lale.lib.lale import ConcatFeatures as Concat
from lale.lib.lale import NoOp

from lale.lib.lale import Hyperopt
import lale.datasets

class TestGrammar(unittest.TestCase):
    def setUp(self):
        (self.train_X, self.train_y), (self.test_X, self.test_y) = lale.datasets.load_iris_df()

    def test_grammar_simple(self):
        g = Grammar()
        g.start       = g.estimator
        g.estimator   = (NoOp | g.transformer) >> g.prim_est
        g.transformer = (NoOp | g.transformer) >> g.prim_tfm

        g.prim_est    = LR | KNN
        g.prim_tfm    = PCA | Scaler

        generated = g.unfold(6)
        sample = g.sample(6)
        
        # unfold and sample return a PlannedOperator
        assert isinstance(generated, PlannedOperator)
        assert isinstance(sample, PlannedOperator)
        
        # test getter for methods other than Nonterminal   
        if isinstance(generated, PlannedPipeline):
            assert (generated._name.startswith('pipeline'))
            
        try:
            gtrainer = Hyperopt(estimator=generated, max_evals=3, scoring='r2')
            gtrained = gtrainer.fit(self.train_X, self.train_y)
            assert isinstance(gtrained.get_pipeline(), TrainedOperator)
        except ValueError:
            # None of the trials succeeded
            pass
        
        try:
            strainer = Hyperopt(estimator=sample, max_evals=3, scoring='r2')
            strained = strainer.fit(self.train_X, self.train_y)
            assert isinstance(strained.get_pipeline(), TrainedOperator)
        except ValueError:
            # None of the trials succeeded
            pass      
        
    def test_grammar_all_combinator(self):
        g = Grammar()

        g.start       = g.estimator
        g.estimator   = g.term_est | g.transformer >> g.term_est
        g.term_est    = g.prim_est | g.ensemble
        g.ensemble    = Boost ( base_estimator = LR )
        g.transformer = g.union_tfm | g.union_tfm >> g.transformer
        g.union_tfm   = g.prim_tfm | g.union_body >> Concat
        g.union_body  = g.transformer | g.transformer & g.union_body
        
        g.prim_est    = LR | KNN
        g.prim_tfm    = PCA | Scaler
        g.ensembler   = Boost

        generated = g.unfold(7)
        sample = g.sample(7)
        assert isinstance(generated, PlannedOperator)
        assert isinstance(sample, PlannedOperator)
        
        # Train
        try:
            gtrainer = Hyperopt(estimator=generated, max_evals=3, scoring='r2')
            gtrained = gtrainer.fit(self.train_X, self.train_y)
            assert isinstance(gtrained.get_pipeline(), TrainedOperator)
        except ValueError:
            # None of the trials succeeded
            pass
        
        try:
            strainer = Hyperopt(estimator=sample, max_evals=3, scoring='r2')
            strained = strainer.fit(self.train_X, self.train_y)
            assert isinstance(strained.get_pipeline(), TrainedOperator)
        except ValueError:
            # None of the trials succeeded
            pass